﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FireBus.Contracts;
using NUnit.Framework;

namespace FireBus.UnitTest.Fixtures
{
	[TestFixture]
	public class ExtensionMetadataTestFixture
	{
		[Test]
		public void FromType_SimpleEventListener_ReturnMetadata()
		{
			var metadata = ExtensionMetadata.FromType(
				"dummy", typeof (SimpleEventListener));
			
			Assert.IsTrue(
				metadata.CanListenToEvent(
					typeof(String)));
		}

		[Test]
		public void FromType_SimpleRequestListener_ReturnMetadata()
		{
			var metadata = ExtensionMetadata.FromType(
				"dummy", typeof(SimpleRequestListener));

			Assert.IsTrue(
				metadata.CanListenToRequest(
					typeof(String),
					typeof(Object)));
		}

		[Test]
		public void FromType_SimpleEventNotifier_ReturnMetadata()
		{
			var metadata = ExtensionMetadata.FromType(
				"dummy", typeof(SimpleEventNotifier));

			Assert.IsTrue(
				metadata.CanNotifyAboutEvent(
					typeof(String)));
		}

		private class SimpleEventListener : IEventListener<String>
		{
			public void OnEvent(string args)
			{
			}
		}

		private class SimpleRequestListener : IRequestListener<String, Object>
		{
			public Object OnRequest(String request)
			{
				throw new NotImplementedException();
			}
		}

		public class SimpleEventNotifier : IEventNotifier<String>
		{
			protected void OnNotify()
			{
				if (Notify != null)
				{
					Notify(
						this,
						new NotifyEventArgs<String>(String.Empty));
				}
			}

			public event NotifyEventHandler<String> Notify;
		}
	}
}
