﻿using System;
using System.Collections.Generic;

using FireBus.Contracts;
using FireBus.UnitTest.Contracts;


namespace FireBus.UnitTest.Fixtures.Hosts
{
	public class LogStorage : MarshalByRefObject, ILogStorage
	{
		private readonly IList<ItemContainer> _Items = new List<ItemContainer>();

		private readonly Object _Sync = new Object();

		public Int32 Count
		{
			get
			{
				lock (_Sync)
				{
					return _Items.Count;
				}
			}
		}

		public LogMessageEvent this[Int32 index]
		{
			get
			{
				lock (_Sync)
				{
					return _Items[index].Message;
				}
			}
		}

		public void Push(ExtensionName name, LogMessageEvent message)
		{
			if (name == null)
			{
				throw new ArgumentNullException("name");
			}
			if (message == null)
			{
				throw new ArgumentNullException("message");
			}
			var item =
				new ItemContainer
					{
						Message = message,
						Name = name
					};
			lock (_Sync)
			{
				_Items.Add(item);
			}
		}

		public Boolean Contains(ExtensionName name, LogMessageEvent message)
		{
			lock (_Sync)
			{
				foreach (var item in _Items)
				{
					if (item.Name.Equals(name))
					{
						var m = item.Message;
						if (m.Message == message.Message &&
							m.Occured == message.Occured &&
							m.Source == message.Source &&
							m.Type == message.Type)
						{
							return true;
						}
					}
				}
			}
			return false;
		}

		public override Object InitializeLifetimeService()
		{
			return null;
		}


		private class ItemContainer
		{
			public ExtensionName Name;
			public LogMessageEvent Message;
		}
	}
}
