﻿using System;
using System.IO;
using System.Linq;
using FireBus.Isolation;
using NUnit.Framework;
using NUnit.Mocks;

using FireBus.Contracts;
using FireBus.Core;


namespace FireBus.UnitTest.Fixtures
{
	[TestFixture]
	public class IsolatedExtensionLoaderTestFixture
	{
		private static Object[] GetExtensionLoadTestCases()
		{
			const String relativePath =
				@"..\..\..\FireBus.UnitTest.Extensions\bin\Debug\";
			const String eventNotifierTypeName =
				"FireBus.UnitTest.Extensions.EventNotifierExtension, FireBus.UnitTest.Extensions, Version=0.1.0.0";
			const String eventListenerTypeName =
				"FireBus.UnitTest.Extensions.EventListenerExtension, FireBus.UnitTest.Extensions, Version=0.1.0.0";
			const String requestSenderTypeName =
				"FireBus.UnitTest.Extensions.StringConverterExtension, FireBus.UnitTest.Extensions, Version=0.1.0.0";
			var absolutePath = Path.GetFullPath(relativePath);
			

			return
				new Object[]
				{
					new Object[] { relativePath, eventNotifierTypeName},
					new Object[] {relativePath, eventListenerTypeName},
					new Object[] { absolutePath, eventNotifierTypeName},
					new Object[] {absolutePath, eventListenerTypeName},
					new Object[] { absolutePath, requestSenderTypeName},
					new Object[] {absolutePath, requestSenderTypeName}
				};
		}

		[Test]
		[TestCaseSource("GetExtensionLoadTestCases")]
		public void Load_Extension_ShouldSucceed(String path, String type)
		{
			var loader = InitializeLoader();
			
			var token =
				new ExtensionToken(
					new ExtensionName("extension"),
					new ExtensionPath(
						path),
					new TypeName(
						type));

			var result = loader.Load(token);

			AssertExtensionLoad(result);
		}

		private static IsolatedExtensionLoader InitializeLoader()
		{
			//Initializing isolated extension loader
			var messageAccessor =
				(IMessageAccessor)new DynamicMock(
					typeof(IMessageAccessor)).MockInstance;
			var extensionAccessor =
				(IExtensionAccessor)new DynamicMock(
					typeof(IExtensionAccessor)).MockInstance;
			var loader = new IsolatedExtensionLoader(
				messageAccessor,
				extensionAccessor);
			return loader;
		}

		private static void AssertExtensionLoad(ExtensionLoadResult result)
		{
			Assert.IsFalse(
				result.Failed,
				"Loading extension should succeed.");
			Assert.NotNull(
				result.Extension,
				"Extension instance should be created.");
			Assert.IsTrue(
				result.Errors.Count() == 0,
				"Error count should be 0");
		}
	}
}
